SystemOrganization addCategory: 'PassportKernel-Gemstone'!

!Class methodsFor: '*passportkernel-gemstone' stamp: 'PaulDeBruicker 03/07/2011 12:39'!
allInstancesDo:aBlock

System commitTransaction.
^self allInstances do: aBlock.! !

ResourcePathFacade subclass: #GemstoneFilenamePathFacade
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PassportKernel-Gemstone'!

!GemstoneFilenamePathFacade methodsFor: 'conversion services' stamp: 'PaulDeBruicker 03/07/2011 14:05'!
constructNativeResourceIDFromResourcePath: aResourcePath
	"Answer whatever value the current Smalltalk execution environment canonically would use to name/identify the external/persistent/remote resouce (e.g, a file) referenced by <aResourcePath> (.e.g, a String in Squeak, a Filename in VisualWorks, etc.)"
^aResourcePath asAbsolute printString.



	! !

!GemstoneFilenamePathFacade methodsFor: 'conversion services' stamp: 'PaulDeBruicker 03/12/2011 11:30'!
constructResourcePathFromNativeResourceID: aNativeResourceID 
	"Construct and answer a ResourcePath that identifies the same resource
	(e.g, a file) as does <aNativeResourceID> (where <aNativeResourceID> is
	an object canonically used by the host Smalltalk implementation to
	name/identify external/persistent/remote resources;) in most Smalltalk
	implementations such an object would be a String.)"

	"{SqueakFilenamePathFacade new constructResourcePathFromNativeResourceID: FileDirectory default pathName}"
	"{SqueakFilenamePathFacade new constructResourcePathFromNativeResourceID: FileDirectory default pathName, FileDirectory pathNameDelimiter asString, 'foo.txt'}"
	"{SqueakFilenamePathFacade new constructResourcePathFromNativeResourceID: 'c:\foo\bar.txt'}"
	"{SqueakFilenamePathFacade new constructResourcePathFromNativeResourceID: '\foo\bar.txt'}"
	"{SqueakFilenamePathFacade new constructResourcePathFromNativeResourceID: 'foo\bar.txt'}"
	"{SqueakFilenamePathFacade new constructResourcePathFromNativeResourceID: 'bar.txt'}"
	"{SqueakFilenamePathFacade new constructResourcePathFromNativeResourceID: '.\bar.txt'}"
	"{SqueakFilenamePathFacade new constructResourcePathFromNativeResourceID: '.'}"
	"{SqueakFilenamePathFacade new constructResourcePathFromNativeResourceID: ''}"

	| ns components |

	"FileDirectory 
		splitName: 'foo\bar.txt' 
		to: [:prefix :suffix | Array with: prefix with: suffix]."

	ServerFileDirectory 
		splitName: aNativeResourceID 
		to: [:prefix :suffix |   
			prefix size = 0
				ifTrue:
					[ns := nil.
					suffix = '.'
							ifTrue: [components := Array new]
							ifFalse: [components := suffix isEmpty ifTrue: [#()] ifFalse: [Array with: suffix]]]
					ifFalse: 
						[| directory isRelative root endsWithColon |
						isRelative := false.
						[directory := FileDirectory forFileName: aNativeResourceID] 
							on: Error 
							do: [:ex | "We assume the error is complaining that the pathname isn't absolute--which means it's a relative path!!"
								isRelative := true. 
								ex return].
						isRelative
							ifTrue:
								[ns := nil.
								components := UtilityFunction componentsFromString: aNativeResourceID separator: self pathnameSeparator]
							ifFalse:
								[components := directory pathParts.
								root := components first.
								endsWithColon := root last = $:.
								
									ns := self rootDirectoryMetavalueString.
								suffix size > 0 ifTrue: [components := components copyWith: suffix]]]].


	^(self newResourcePath)
		setHostname: #localhost;
		setNamespace: ns;
		setComponents: components;
		beImmutable! !

!GemstoneFilenamePathFacade methodsFor: 'directory services' stamp: 'PaulDeBruicker 03/06/2011 16:43'!
copyResource: sourceOperand to: destinationOperand
	"Copy the contents of the resource (e.g., a file) named/identified by <sourceOperand> to <destinationOperand>.  If the resource named/identified by <destinationOperand> does not exist, create it. If a resource so-named already exists, make it have exactly the same contents as the resource named/identified by <sourceNativeFileID>."

	| sourceStream targetStream buffer |
	sourceStream := self newExternalReadStream: sourceOperand type: #binary.
	[targetStream := self newExternalWriteStream: destinationOperand mode: #truncate check: false type: #binary.
	[buffer := String new: 50000.
	[sourceStream atEnd] whileFalse:
		[targetStream nextPutAll: (sourceStream nextInto: buffer)]] ensure: [targetStream close]] ensure: [sourceStream close].
! !

!GemstoneFilenamePathFacade methodsFor: 'accessing' stamp: 'PaulDeBruicker 03/12/2011 11:30'!
defaultDirectoryPathnameString
	"Answer a String that names/identifies the pathname of the external-file namespace relative to which non-absolute filenames will be interpreted.  In more conventional terminology, answer a String naming/identifying the actual full absolute pathname of the current default directory/folder.  It is acceptable, but not ideal, to answer a metavalue that serves as an indirect reference to the default directory (e.g., '.' on UNIX/Windows systems, and ':' on the Macintosh.) If not answering a metavalue, the answered String must start with the root directory of the current default volume/device (e.g., '/' on a Unix system, 'C:\' or 'D:\' (or whatever volume is the default) on a Windows system.)" 

	^ServerFileDirectory default pathName! !

!GemstoneFilenamePathFacade methodsFor: 'directory services' stamp: 'PaulDeBruicker 03/12/2011 11:30'!
deleteResource: operand
	"Delete the resource (e.g., a file) named/identified by <operand>. If the resource named/identified by <operand> does not exist, raise an exception."

	| directory suffix |
	directory := ServerFileDirectory forFileName: operand.
	suffix := ServerFileDirectory localNameFor: operand.
	^(directory directoryExists: suffix)
		ifTrue: [(ServerFileDirectory on: operand) recursiveDelete]
		ifFalse: [directory deleteFileNamed: suffix]
! !

!GemstoneFilenamePathFacade methodsFor: 'accessing' stamp: 'PaulDeBruicker 03/12/2011 11:31'!
fileExtensionSeparator
	"Answer the character that is used to separate file extensions from the rest of the file's name."

	^ServerFileDirectory extensionDelimiter! !

!GemstoneFilenamePathFacade methodsFor: 'directory services' stamp: 'PaulDeBruicker 03/12/2011 11:31'!
makeDirectory: operand
	"Create a directory named <operand>. If the resource named/identified by <operand> already exists, raise an exception. A 'directory'  is a resource that serves as a namespace containing other resources."

	^(ServerFileDirectory on: operand) assureExistence! !

!GemstoneFilenamePathFacade methodsFor: 'directory services' stamp: 'PaulDeBruicker 03/12/2011 11:31'!
moveResource: sourceOperand to: destinationOperand
	"Move the resource (e.g., a file) named/identified by <sourceOperand> to <destinationOperand>. If possible, implement the 'move' by removing and adding directory entries, instead of by actually copying the contents of the source resource. However, if that is not possible, then a) copy <sourceOperand> to <destinationOperand>, and then b) delete <sourceOperand>."

	| sourceDirectory targetDirectory | 
	sourceDirectory := ServerFileDirectory forFileName: sourceOperand.
	targetDirectory := ServerFileDirectory forFileName: destinationOperand.
	^sourceDirectory pathName = targetDirectory pathName
		ifTrue: [sourceDirectory rename: (ServerFileDirectory localNameFor: sourceOperand) toBe: (ServerFileDirectory localNameFor: destinationOperand)]
		ifFalse:
			[self copyResource: sourceOperand to: destinationOperand.
			self deleteResource: sourceOperand]

	

! !

!GemstoneFilenamePathFacade methodsFor: 'directory services' stamp: 'PaulDeBruicker 03/14/2011 11:40'!
newExternalReadStream: operand type: fileType
	"Synopsis

		Returns an external readStream that reads from the external data source with the given name.

	Specification (based on ANSI Smalltalk standard)

		Locate an external data source that is identified by the value of <operand>. The <operand> parameter is an instance of whatever class the host Smalltalk implementation would canonically use to identify an external data source (e.g., a Strring, a Filename, a URL, etc.)  If <operand> is a String (or if its content requires syntactical interpretation,) then its syntax is implementation-dependent.

		Return an object conforming to the ANSI <readFileStream> protocol whose future sequence values initially consist of the elements of the external data source and which initially has no past sequence values. The ordering of the sequence values is the same as the ordering within the external data source. The external data source serves as the stream backing store of the returned object. The value of <fileType> determines the external stream type and sequence value type of the result object.

	Parameters

		operand 		<readableString> 	identifies the external data source (typically, but not necessarily, a disk file)
		fileType		<symbol> 			an enumeration constant that specifies the type of the external data source, as defined by the ANSI Smalltalk standard

	Return Value

		<readFileStream> new

	Errors

		It is an error if the data source identified by <operand> does not exist, or if the user does not have read access permission"
|file|

	file:=GemstoneServerFileStream readingFromFileNamed: operand.

	^file

	! !

!GemstoneFilenamePathFacade methodsFor: 'directory services' stamp: 'PaulDeBruicker 03/07/2011 17:43'!
newExternalWriteStream: operand mode: mode check: enforceExistentialConstraint type: fileType
	"Synopsis

		Returns an external writeStream that writes to the external data source with the given name.

	Specification (based on ANSI Smalltalk standard)

		Depending upon the values of <enforceExistentialConstraint> and <mode>, either a) create a new external data source, or b) locate an existing external data source, which in either case is identified by the value of <operand>. The <operand> parameter is an instance of whatever class the host Smalltalk implementation would canonically use to identify an external data source (e.g., a Strring, a Filename, a URL, etc.)  If <operand> is a String (or if its content requires syntactical interpretation,) then its syntax is implementation-dependent.

		Return an object conforming to the ANSI Smalltalk <writeFileStream> protocol. The external data source serves as the stream backing store of the returned object. The returned object is a write-back stream. The value of <fileType> determines the external stream type and sequence value type of the result object.

		Valid values for <mode> are: #create, #append, and #truncate. The meaning of these values are:

				#create create a new external data source, with initial position at the beginning
				#append use an existing external data source, with initial position at its end
				#truncate use an existing external data source, initially truncating it.

		The value of <mode> determines the initial state of the past sequence values and future sequence values of the result object. If <mode> is #create or #truncate the past sequence values and future sequence values are both initially empty. If <mode> is #append the past sequence values initially consist of the elements of the external data source and future sequence values is initially empty. The ordering of the sequence values is the same as the ordering within the external data source. The <enforceExistentialConstraint> flag determines whether the data source specified by <operand> must exist or not exist.

		If <mode> = #create and <enforceExistentialConstraint> = false and the data source exists, then the existing file is used.
		If <mode> = #append and <enforceExistentialConstraint> = false and the data source does not exist, then it is created.
		If <mode> = #truncate and <enforceExistentialConstraint> = false and the data source  does not exist, then it is created.

		This operation is undefined if a value other than #create, #append or #truncate is used as the <mode> argument.

	Parameters

		operand 						<readableString> 	identifies the external data source (typically, but not necessarily, a disk file)
		mode 						<symbol> 			an enumeration constant that specifies the initial state of the external stream, as defined by the ANSI Smalltalk standard
		enforceExistentialConstraint 	<boolean> 			specifies whether or not the existence or non-existence of the data source identified by <operand> is an error
		fileType						<symbol> 			an enumeration constant that specifies the type of the external data source, as defined by the ANSI Smalltalk standard

	Return Value

		<writeFileStream> new

	Errors

		If <mode> = #create and <enforceExistentialConstraint> = true and the file exists.
		If <mode> = #append and <enforceExistentialConstraint> = true and the file does not exist.
		If <mode> = #truncate and <enforceExistentialConstraint> = true and the file does not exist.
		If the user does not have write permissions for the external data source.
		If the user does not have creation permissions for an external data source that is to be created."

	^(enforceExistentialConstraint 
		ifTrue: 
			[mode == #truncate
				ifTrue: 
					[(self resourceExists: operand) ifFalse: [^self signalError: 'File must exist in order to be truncated.'].
					GsFile openWrite: operand]
				ifFalse:
					[mode == #create
						ifTrue: [GsFile openWrite: operand]
						ifFalse: [GsFile openAppend: operand]]]
		ifFalse: 
			[mode == #truncate 
				ifTrue: [GsFile openWrite: operand] 
				ifFalse: [mode == #create
						ifTrue: [GsFile openWrite: operand]
						ifFalse: [GsFile openAppend: operand]]]) 
					perform: fileType! !

!GemstoneFilenamePathFacade methodsFor: 'accessing' stamp: 'PaulDeBruicker 03/12/2011 11:32'!
pathnameSeparator
	"Answer the character that is used to separate file pathname components (in other words, the directory separator character.)"

	^ServerFileDirectory pathNameDelimiter! !

!GemstoneFilenamePathFacade methodsFor: 'directory services' stamp: 'PaulDeBruicker 03/06/2011 16:43'!
renameResource: originalOperand to: newOperand
	"Rename the resource (e.g., a file) named/identified by <originalOperand> to <newOperand>"

	^self moveResource: originalOperand to: newOperand! !

!GemstoneFilenamePathFacade methodsFor: 'directory services' stamp: 'PaulDeBruicker 03/12/2011 11:32'!
resourceExists: operand
	"Answer whether the resource (e.g., a file) named/identified by <operand> exists"

	| directory  |
	directory := ServerFileDirectory forFileName: operand.
	^directory exists and: [directory fileOrDirectoryExists: (ServerFileDirectory localNameFor: operand)].
	
	
	! !

!GemstoneFilenamePathFacade methodsFor: 'directory services' stamp: 'PaulDeBruicker 03/12/2011 11:32'!
resourceIsDirectory: operand
	"Answer whether the resource (e.g., a file) named/identified by <operand> is a directory.  A resource is a directory if it acts as a namespace containing other resources."

	| directory |
	directory := ServerFileDirectory forFileName: operand.
	^directory exists and: [directory directoryExists: (ServerFileDirectory localNameFor: operand)]! !

!GemstoneFilenamePathFacade methodsFor: 'conversion services' stamp: 'PaulDeBruicker 03/07/2011 14:01'!
rootDirectoryMetavalueString
	^'./'! !

!GemstoneFilenamePathFacade methodsFor: 'directory services' stamp: 'PaulDeBruicker 03/12/2011 11:32'!
sizeOfResource: operand
	"Answer the size (in bytes) of the resource (e.g., a file) named/identified by <operand>"

	| directory |
	directory := ServerFileDirectory forFileName: operand.
	directory exists ifFalse: [^0].
	^(directory entryAt: (ServerFileDirectory localNameFor: operand) ifAbsent: [^0]) fileSize

! !

!GemstoneFilenamePathFacade methodsFor: 'testing' stamp: 'PaulDeBruicker 03/06/2011 16:43'!
syntaxPermitsHostname
	^false! !

!GemstoneFilenamePathFacade methodsFor: 'directory services' stamp: 'PaulDeBruicker 03/12/2011 11:33'!
withResourcePathsOfResourcesInDirectory: operand do: block1
	"For each external/persistent/remote resource (e.g., a file) contained by the resource directory named/identified by <operand>, evaluate the one-argument Block <block1> with an argument that is a ResourcePath (which see) naming/identifying the external/persistent/remote resource (e.g, a file.) If <operand> does not name/identify a directory, raise an exception."

	| rpDirectory |
	rpDirectory := self constructResourcePathFromNativeResourceID: operand.
	(ServerFileDirectory on: operand) fileAndDirectoryNames do: [:suffix | block1 value: (rpDirectory appending: suffix)]! !

!FileDirectory class methodsFor: '*passportkernel-gemstone' stamp: 'PaulDeBruicker 03/07/2011 12:39'!
extensionDelimiter
	"Return the character used to delimit filename extensions on this platform. Most platforms use the period (.) character."

	^ $.
! !

!FileDirectory methodsFor: '*passportkernel-gemstone' stamp: 'PaulDeBruicker 03/07/2011 18:47'!
fileOrDirectoryExists: aPath
	^(self fileExists: aPath) or: [self directoryExists: aPath]! !

!ServerFileDirectory classSide methodsFor: '*passportkernel-gemstone' stamp: 'PaulDeBruicker 03/12/2011 11:51'!
extensionDelimiter
	"Return the character used to delimit filename extensions on this platform. Most platforms use the period (.) character."

	^ $.
! !

UrlPathFacade subclass: #GemstoneUrlPathFacade
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PassportKernel-Gemstone'!

GemstoneUrlPathFacade subclass: #GemstoneAbstractFileUrlFacade
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PassportKernel-Gemstone'!

!GemstoneAbstractFileUrlFacade methodsFor: 'testing' stamp: 'PaulDeBruicker 03/06/2011 16:44'!
syntaxPermitsUserCredentials
	^true! !

GemstoneAbstractFileUrlFacade subclass: #GemstoneFileUrlFacade
	instanceVariableNames: 'platformFacade'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PassportKernel-Gemstone'!

!GemstoneFileUrlFacade methodsFor: 'directory services' stamp: 'PaulDeBruicker 03/06/2011 16:45'!
copyResource: sourceOperand to: destinationOperand 
	"Copy the contents of the resource (e.g., a file) named/identified by <sourceOperand> to <destinationOperand>.  If the resource named/identified by <destinationOperand> does not exist, create it. If a resource so-named already exists, make it have exactly the same contents as the resource named/identified by <sourceNativeFileID>."

	^self platformFacade copyResource: sourceOperand to: destinationOperand
	! !

!GemstoneFileUrlFacade methodsFor: 'directory services' stamp: 'PaulDeBruicker 03/06/2011 16:45'!
deleteResource: operand
	"Delete the resource (e.g., a file) named/identified by <operand>. If the resource named/identified by <operand> does not exist, raise an exception."

	^self platformFacade deleteResource: operand! !

!GemstoneFileUrlFacade methodsFor: 'stream creation' stamp: 'PaulDeBruicker 03/06/2011 16:45'!
newExternalReadStream: operand type: fileType
	"Synopsis

		Returns an external readStream that reads from the external data source with the given name.

	Specification (based on ANSI Smalltalk standard)

		Locate an external data source that is identified by the value of <operand>. The <operand> parameter is an instance of whatever class the host Smalltalk implementation would canonically use to identify an external data source (e.g., a Strring, a Filename, a URL, etc.)  If <operand> is a String (or if its content requires syntactical interpretation,) then its syntax is implementation-dependent.

		Return an object conforming to the ANSI <readFileStream> protocol whose future sequence values initially consist of the elements of the external data source and which initially has no past sequence values. The ordering of the sequence values is the same as the ordering within the external data source. The external data source serves as the stream backing store of the returned object. The value of <fileType> determines the external stream type and sequence value type of the result object.

	Parameters

		operand 		<readableString> 	identifies the external data source (typically, but not necessarily, a disk file)
		fileType		<symbol> 			an enumeration constant that specifies the type of the external data source, as defined by the ANSI Smalltalk standard

	Return Value

		<readFileStream> new

	Errors

		It is an error if the data source identified by <operand> does not exist, or if the user does not have read access permission"

	^self platformFacade newExternalReadStream: operand type: fileType! !

!GemstoneFileUrlFacade methodsFor: 'stream creation' stamp: 'PaulDeBruicker 03/06/2011 16:45'!
newExternalWriteStream: operand mode: mode check: enforceExistentialConstraint type: fileType
	"Synopsis

		Returns an external writeStream that writes to the external data source with the given name.

	Specification (based on ANSI Smalltalk standard)

		Depending upon the values of <enforceExistentialConstraint> and <mode>, either a) create a new external data source, or b) locate an existing external data source, which in either case is identified by the value of <operand>. The <operand> parameter is an instance of whatever class the host Smalltalk implementation would canonically use to identify an external data source (e.g., a Strring, a Filename, a URL, etc.)  If <operand> is a String (or if its content requires syntactical interpretation,) then its syntax is implementation-dependent.

		Return an object conforming to the ANSI Smalltalk <writeFileStream> protocol. The external data source serves as the stream backing store of the returned object. The returned object is a write-back stream. The value of <fileType> determines the external stream type and sequence value type of the result object.

		Valid values for <mode> are: #create, #append, and #truncate. The meaning of these values are:

				#create create a new external data source, with initial position at the beginning
				#append use an existing external data source, with initial position at its end
				#truncate use an existing external data source, initially truncating it.

		The value of <mode> determines the initial state of the past sequence values and future sequence values of the result object. If <mode> is #create or #truncate the past sequence values and future sequence values are both initially empty. If <mode> is #append the past sequence values initially consist of the elements of the external data source and future sequence values is initially empty. The ordering of the sequence values is the same as the ordering within the external data source. The <enforceExistentialConstraint> flag determines whether the data source specified by <operand> must exist or not exist.

		If <mode> = #create and <enforceExistentialConstraint> = false and the data source exists, then the existing file is used.
		If <mode> = #append and <enforceExistentialConstraint> = false and the data source does not exist, then it is created.
		If <mode> = #truncate and <enforceExistentialConstraint> = false and the data source  does not exist, then it is created.

		This operation is undefined if a value other than #create, #append or #truncate is used as the <mode> argument.

	Parameters

		operand 					<readableString> 	identifies the external data source (typically, but not necessarily, a disk file)
		mode 						<symbol> 			an enumeration constant that specifies the initial state of the external stream, as defined by the ANSI Smalltalk standard
		enforceExistentialConstraint 	<boolean> 			specifies whether or not the existence or non-existence of the data source identified by <operand> is an error
		fileType						<symbol> 			an enumeration constant that specifies the type of the external data source, as defined by the ANSI Smalltalk standard

	Return Value

		<writeFileStream> new

	Errors

		If <mode> = #create and <enforceExistentialConstraint> = true and the file exists.
		If <mode> = #append and <enforceExistentialConstraint> = true and the file does not exist.
		If <mode> = #truncate and <enforceExistentialConstraint> = true and the file does not exist.
		If the user does not have write permissions for the external data source.
		If the user does not have creation permissions for an external data source that is to be created."

	^self platformFacade newExternalWriteStream: operand mode: mode check: enforceExistentialConstraint type: fileType! !

!GemstoneFileUrlFacade methodsFor: 'conversion services' stamp: 'PaulDeBruicker 03/06/2011 16:45'!
operandFromResourcePath: aResourcePath
	| stream |
	stream := (String new: 64) writeStream.
	self platformFacade print: aResourcePath on: stream.
	^stream contents! !

!GemstoneFileUrlFacade methodsFor: 'accessing' stamp: 'PaulDeBruicker 03/06/2011 16:45'!
platformFacade
	platformFacade == nil ifTrue: [platformFacade := self resourcePathContext facadeAt: #platform].
	^platformFacade! !

!GemstoneFileUrlFacade methodsFor: 'accessing' stamp: 'PaulDeBruicker 03/06/2011 16:45'!
protocol
	^#file! !

!GemstoneFileUrlFacade methodsFor: 'directory services' stamp: 'PaulDeBruicker 03/06/2011 16:45'!
renameResource: originalOperand to: newOperand
	"Rename the resource (e.g., a file) named/identified by <originalOperand> to <newOperand>"

	^self platformFacade renameResource: originalOperand to: newOperand! !

!GemstoneFileUrlFacade methodsFor: 'directory services' stamp: 'PaulDeBruicker 03/06/2011 16:45'!
resourceExists: operand
	"Answer whether the resource (e.g., a file) named/identified by <operand> exists"
	
	^self platformFacade resourceExists: operand

	! !

!GemstoneFileUrlFacade methodsFor: 'directory services' stamp: 'PaulDeBruicker 03/06/2011 16:45'!
resourceIsDirectory: operand
	"Answer whether the resource (e.g., a file) named/identified by <operand> is a directory.  A resource is a directory if it acts as a namespace containing other resources."

	^self platformFacade resourceIsDirectory: operand! !

!GemstoneFileUrlFacade methodsFor: 'directory services' stamp: 'PaulDeBruicker 03/06/2011 16:45'!
sizeOfResource: operand
	"Answer the size (in bytes) of the resource (e.g., a file) named/identified by <operand>"

	^self platformFacade sizeOfResource: operand! !

!GemstoneFileUrlFacade methodsFor: 'testing' stamp: 'PaulDeBruicker 03/06/2011 16:45'!
syntaxPermitsPort
	^false! !

!GemstoneFileUrlFacade methodsFor: 'directory services' stamp: 'PaulDeBruicker 03/06/2011 16:45'!
withResourcePathsOfResourcesInDirectory: operand do: block1
	"For each external/persistent/remote resource (e.g., a file) contained by the resource directory named/identified by <operand>, evaluate the one-argument Block <block1> with an argument that is a ResourcePath (which see) naming/identifying the external/persistent/remote resource (e.g, a file.) If <operand> does not name/identify a directory, raise an exception."

	^self platformFacade withResourcePathsOfResourcesInDirectory: operand do: block1! !

GemstoneUrlPathFacade subclass: #GemstoneHttpUrlFacade
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PassportKernel-Gemstone'!

!GemstoneHttpUrlFacade methodsFor: 'private' stamp: 'PaulDeBruicker 03/06/2011 16:53'!
configureRequest: anHttpRequest for: aResourcePath

	anHttpRequest resourcePath: aResourcePath.
	(HTTPSocket shouldUseProxy: aResourcePath hostname)
		ifTrue: [anHttpRequest proxy: (GemstoneNetworkCorrespondent hostname: HTTPSocket httpProxyServer port: HTTPSocket httpProxyPort)].
	^anHttpRequest
	! !

!GemstoneHttpUrlFacade methodsFor: 'stream creation' stamp: 'PaulDeBruicker 03/06/2011 16:45'!
newExternalReadStream: operand type: fileType 
	"Synopsis

		Returns an external readStream that reads from the external data source with the given name.

	Specification (based on ANSI Smalltalk standard)

		Locate an external data source that is identified by the value of <operand>. The <operand> parameter is an instance of whatever class the host Smalltalk implementation would canonically use to identify an external data source (e.g., a Strring, a Filename, a URL, etc.)  If <operand> is a String (or if its content requires syntactical interpretation,) then its syntax is implementation-dependent.

		Return an object conforming to the ANSI <readFileStream> protocol whose future sequence values initially consist of the elements of the external data source and which initially has no past sequence values. The ordering of the sequence values is the same as the ordering within the external data source. The external data source serves as the stream backing store of the returned object. The value of <fileType> determines the external stream type and sequence value type of the result object.

	Parameters

		operand 	<readableString> 	identifies the external data source (typically, but not necessarily, a disk file)
		fileType		<symbol> 			an enumeration constant that specifies the type of the external data source, as defined by the ANSI Smalltalk standard

	Return Value

		<readFileStream> new

	Errors

		It is an error if the data source identified by <operand> does not exist, or if the user does not have read access permission"
		
	^(self configureRequest: HttpGetRequest new for: operand) responseFromServer readStream
! !

!GemstoneHttpUrlFacade methodsFor: 'conversion services' stamp: 'PaulDeBruicker 03/06/2011 16:45'!
operandFromResourcePath: aResourcePath
	^aResourcePath withoutFragment! !

!GemstoneHttpUrlFacade methodsFor: 'accessing' stamp: 'PaulDeBruicker 03/06/2011 16:45'!
protocol
	^#http! !

!GemstoneHttpUrlFacade methodsFor: 'directory services' stamp: 'PaulDeBruicker 03/06/2011 16:45'!
resourceExists: operand
	"Answer whether the resource (e.g., a file) named/identified by <operand> exists"

	| request |
	request := self configureRequest: HttpHeadRequest new for: operand.
	^(request basicResponseFromServer) isError not
! !

!GemstoneHttpUrlFacade methodsFor: 'directory services' stamp: 'PaulDeBruicker 03/06/2011 16:45'!
sizeOfResource: operand
	"Answer the size (in bytes) of the resource (e.g., a file) named/identified by <operand>"
	
	| request response |
	request := self configureRequest: HttpHeadRequest new for: operand.
	response := request responseFromServer.
	^response contentLength


	! !

!GemstoneHttpUrlFacade methodsFor: 'testing' stamp: 'PaulDeBruicker 03/06/2011 16:45'!
syntaxPermitsFragment
	^true! !

!GemstoneHttpUrlFacade methodsFor: 'testing' stamp: 'PaulDeBruicker 03/06/2011 16:45'!
syntaxPermitsQueryString
	^true! !

!GemstoneUrlPathFacade methodsFor: 'conversion services' stamp: 'PaulDeBruicker 03/06/2011 16:44'!
constructNativeResourceIDFromString: aString
	"Answer whatever value the current Smalltalk execution environment canonically would use to name/identify the external/persistent/remote resource (e.g, a file) referenced by <aString> (.e.g, a String in Squeak, a Filename in VisualWorks, etc.)"
	
	^aString asUrl

	! !

!GemstoneUrlPathFacade methodsFor: 'conversion services' stamp: 'PaulDeBruicker 03/06/2011 16:44'!
constructResourcePathFromNativeResourceID: aNativeResourceID
	"Construct and answer a ResourcePath that identifies the same resource (e.g, a file) as does <aNativeResourceID> (where <aNativeResourceID> is an object canonically used by the host Smalltalk implementation to name/identify external/persistent/remote resources;) in most Smalltalk implementations such an object would be a String.)"

	^ResourcePath 
		fromString: aNativeResourceID toText 
		withContext: self resourcePathContext! !

!GemstoneUrlPathFacade methodsFor: 'stream creation' stamp: 'PaulDeBruicker 03/06/2011 16:44'!
newExternalWriteStream: operand mode: mode check: enforceExistentialConstraint type: fileType
	^self shouldNotImplement! !

Immutable subclass: #GemstoneNetworkCorrespondent
	instanceVariableNames: 'hostname port'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PassportKernel-Gemstone'!

!GemstoneNetworkCorrespondent classSide methodsFor: 'instance creation' stamp: 'PaulDeBruicker 03/06/2011 16:48'!
hostname: hostname port: port
	^self new
		setHostname: hostname;
		setPort: port;
		beImmutable! !

!GemstoneNetworkCorrespondent methodsFor: 'stream creation' stamp: 'PaulDeBruicker 03/06/2011 16:48'!
connectionStreamDo: block1
	| stream |
	stream := self connectionStream.
	^[block1 value: stream] ensure: [stream close]! !

!GemstoneNetworkCorrespondent methodsFor: 'accessing' stamp: 'PaulDeBruicker 03/06/2011 16:48'!
hostname
	^hostname! !

!GemstoneNetworkCorrespondent methodsFor: 'accessing' stamp: 'PaulDeBruicker 03/06/2011 16:48'!
port
	^port! !

!GemstoneNetworkCorrespondent methodsFor: 'initialize-release' stamp: 'PaulDeBruicker 03/06/2011 16:48'!
setHostname: aHostname
	self assertMutability.
	hostname := aHostname == nil ifFalse: [aHostname asSymbol]! !

!GemstoneNetworkCorrespondent methodsFor: 'initialize-release' stamp: 'PaulDeBruicker 03/06/2011 16:48'!
setPort: portNumber
	self assertMutability.
	port := portNumber! !

SpFileStream subclass: #GemstoneServerFileStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PassportKernel-Gemstone'!

!GemstoneServerFileStream methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 03/14/2011 11:39'!
readingFromFileNamed: aString 
	"^self
I initialize myself to read from a file named aString."

	filename := aString.
	underlyingStream := GsFile open: aString mode: 'r' onClient: ServerFileDirectory onClient.
	^self
! !

!GemstoneServerFileStream methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 03/14/2011 11:39'!
readingFromFilename: anSpFilename 
	filename := anSpFilename asString.
	underlyingStream := BinaryOrTextFile open: anSpFilename filenameString mode: 'r' onClient: ServerFileDirectory onClient.
	^self

! !

!GemstoneServerFileStream methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 03/14/2011 11:40'!
writingToFileNamed: aString 
	"^self
I initialize myself to write to a file named aString."

	filename := aString.
	underlyingStream := GsFile open: aString mode: 'w' onClient: ServerFileDirectory onClient.
	^self
! !

!GemstoneServerFileStream methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 03/14/2011 11:39'!
writingToFilename: anSpFilename 
	filename := anSpFilename asString.
	underlyingStream := GsFile open: anSpFilename filenameString mode: 'w' onClient: ServerFileDirectory onClient.
	^self

! !

!SpFileStream methodsFor: '*passportkernel-gemstone' stamp: 'PaulDeBruicker 08/24/2011 10:02'!
isStream 
	^true! !

!SpFileStream methodsFor: '*passportkernel-gemstone' stamp: 'PaulDeBruicker 03/07/2011 20:55'!
peekFor: anObject 	"Answer false and do not move over the next element if it is not equal to 	the argument, anObject, or if the receiver is at the end. Answer true 	and increment the position for accessing elements, if the next element is 	equal to anObject."	| nextObject |	self atEnd ifTrue: [^false].	nextObject := self next.	"peek for matching element"	anObject = nextObject ifTrue: [^true].	"gobble it if found"	self position: self position - 1.	^false! !

!SpFileStream methodsFor: '*passportkernel-gemstone' stamp: 'PaulDeBruicker 08/24/2011 08:31'!
readStream
	^self contentsStream! !

EnvironmentFacade subclass: #GemstoneEnvironmentFacade
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PassportKernel-Gemstone'!

!GemstoneEnvironmentFacade classSide methodsFor: 'private' stamp: 'PaulDeBruicker 03/06/2011 12:20'!
basicRegisterForStartupNotification
	"Make it so that whenever the Smalltalk image is restarted after a snapshot, the receiver will be sent the message #handleRestart.

	NOTE: This method is intended for use by image-based Smalltalk implementations.  It may or may not have any utility in the case of non-image-based Smalltalk systems."

	Smalltalk addToStartUpList: self! !

!GemstoneEnvironmentFacade classSide methodsFor: 'private' stamp: 'PaulDeBruicker 03/06/2011 12:20'!
basicUnregisterForStartupNotification
	"Make it so that whenever the Smalltalk image is restarted after a snapshot, the receiver will NOT be sent the message #install (nor otherwise notified.)

	NOTE: This method is intended for use by image-based Smalltalk implementations.  It may or may not have any utility in the case of non-image-based Smalltalk systems."

	Smalltalk removeFromStartUpList: self! !

!GemstoneEnvironmentFacade classSide methodsFor: 'private' stamp: 'PaulDeBruicker 03/09/2011 20:30'!
defineScaledDecimalInGlobalNamespace
	"EnvironmentFacade current defineScaledDecimalInGlobalNamespace"

"	| nativeScaledDecimalClass |
	nativeScaledDecimalClass := self nativeScaledDecimalClass.
	nativeScaledDecimalClass == nil ifTrue: [^self].
	self
		defineVariable: #ScaledDecimal
		in: self globalNamespace
		withValue: nativeScaledDecimalClass
		constant: true
		category: #'Chronos-Aliases'
		rebindIfPresent: false
		onCollision: []."! !

!GemstoneEnvironmentFacade classSide methodsFor: 'private' stamp: 'PaulDeBruicker 03/09/2011 20:34'!
defineVariable: key in: aNamespace withValue: value constant: constant category: category rebindIfPresent: rebindIfPresent onCollision: collisionAction 
	"Define a binding (key->value association) in the namespace <aNamespace> (which may just be a Dictionary,) whose key is <key>, and whose value is <value>.  If <constant> is true (and if constant/immutable bindings are supported by the functionality of <aNamespace>,) make the binding constant (immutable.) If supported by the functionality of <aNamespace>, place the binding in the semantic category <category>.  

	If there already exists a binding in <aNamespace> with the given <key>, then:

		1) If the value associated with <key> is the same object as <value>, then just answer <value>.  
		2) If the value associated with <key> is nil, then:
			If there is also a binding in the Undeclared namespace whose key is <key>, set <value> as the value of the binding in the Undeclared namespace, and then move the binding (which might be an Association instance, or else an object that supports Association-like behavior and/or protocol) from the Undeclared namespace to <namespace>. Otherwise, set <value> as the value of the binding already present in <aNamespace>.
		3) If the value associated with <key> is neither nil nor the same object as <value>, but if <rebindIfPresent> is true, then remove any binding whose key is <key> from the Undeclared namespace, and reset the value of the binding in <aNamespace> whose key is <key> to the value <value>.
		4) If the value associated with <key> is neither nil nor the same object as <value>, and if <rebindIfPresent> is also false, then do not change the value of the existing binding, but instead invoke the zero-argument block <collisionAction> (and answer the result.)"

	| presentValue |
	presentValue := aNamespace at: key ifAbsent: [self notAValue].
	^presentValue == value 
		ifTrue: [value]
		ifFalse: 
			[(rebindIfPresent or: [presentValue == self notAValue or: [presentValue == nil]]) 
				ifTrue: 
					[presentValue == self notAValue 
						ifTrue: 
							["self halt."
							(UndefinedSymbols includesKey: key) 
								ifTrue: [aNamespace declare: key from: UndefinedSymbols]]
						ifFalse: [UndefinedSymbols removeKey: key ifAbsent: []].
					aNamespace at: key put: value]
				ifFalse: [collisionAction value]]
! !

!GemstoneEnvironmentFacade classSide methodsFor: 'private' stamp: 'PaulDeBruicker 03/07/2011 11:19'!
globalNamespace
	"Answer the namespace canonically (or most generally) used for the purpose of defining and accessing classes by the host Smalltalk implementation."

	^UserGlobals! !

!GemstoneEnvironmentFacade classSide methodsFor: 'private' stamp: 'PaulDeBruicker 03/07/2011 11:20'!
globalNamespaceName
	"Answer the name of the namespace canonically (or most generally) used for the purpose of defining and accessing classes by the host Smalltalk implementation."

	^#UserGlobals! !

!GemstoneEnvironmentFacade classSide methodsFor: 'private' stamp: 'PaulDeBruicker 03/06/2011 16:42'!
initializeResourcePathFacadeFactoryReferences

	"{EnvironmentFacade current initializeResourcePathFacadeFactoryReferences}"

	super initializeResourcePathFacadeFactoryReferences.
	ResourcePathFacadeFactoryReferences 
		at: #platform put: (Pathname components: #(#GemstoneFilenamePathFacade)); 
		at: #http put: (Pathname components: #(#GemstoneHttpUrlFacade)); 
		at: #file put: (Pathname components: #(#GemstoneFileUrlFacade))! !

!GemstoneEnvironmentFacade classSide methodsFor: 'accessing' stamp: 'PaulDeBruicker 03/06/2011 16:41'!
key
	^#Gemstone! !

!GemstoneEnvironmentFacade classSide methodsFor: 'accessing' stamp: 'PaulDeBruicker 03/07/2011 20:46'!
nativeScaledDecimalClass
	"Answer the class that implements the ANSI-Smallltalk <ScaledDecimal> protocol in the native Smalltalk environment.  Answer nil if none."

	^Globals at: #ScaledDecimal! !

!GemstoneEnvironmentFacade classSide methodsFor: 'accessing' stamp: 'PaulDeBruicker 03/09/2011 19:05'!
newWeakDictionary
	
	^IdentityDictionary new! !

!GemstoneEnvironmentFacade classSide methodsFor: 'accessing' stamp: 'PaulDeBruicker 03/07/2011 12:19'!
purgeUnusedBindingsFromUndeclaredNamespace
	! !

!GemstoneEnvironmentFacade classSide methodsFor: 'private' stamp: 'PaulDeBruicker 03/06/2011 12:20'!
setPlatformClass
	self beCurrent! !

!GemstoneEnvironmentFacade classSide methodsFor: 'image restart' stamp: 'PaulDeBruicker 03/06/2011 12:20'!
startUp
	self handleRestart! !

!GemstoneEnvironmentFacade classSide methodsFor: 'accessing' stamp: 'PaulDeBruicker 03/06/2011 12:20'!
versionNumber
	"EnvironmentFacade current versionNumber"

	| stream major minor |
	stream := Smalltalk version readStream.
	UtilityFunction skipOver: stream until: [:ch | ch isDigit].
	major := stream upTo: $..
	minor := UtilityFunction nextFrom: stream while: [:ch | ch isDigit].
	^Pathname components: (Array with: major asNumber with: minor asNumber)! !
